using System;
using Vintagestory.API.Client;

namespace NaturalNight {
	public class NaturalNightRenderer: IRenderer {
		protected const float graceDegrees = 5f;

		public double RenderOrder => 0;
		public int RenderRange => 0;

		protected ICoreClientAPI capi;

		private float[] lastRgb;

		public NaturalNightRenderer(ICoreClientAPI capi) {
			this.capi = capi;
			
			this.lastRgb = new float[3];
		}

		public void Dispose() {}

		public void OnRenderFrame(float deltaTime, EnumRenderStage stage) {
			if(this.capi.IsGamePaused
					|| stage != EnumRenderStage.Before) {
				return;
			}

			if(this.capi.World.Calendar.ReflectColor.R == this.lastRgb[0]
					&& this.capi.World.Calendar.ReflectColor.G == this.lastRgb[1]
					&& this.capi.World.Calendar.ReflectColor.B == this.lastRgb[2]) {
				// Prevent multiple passes on a single frame.
				return;
			}

			if(this.capi.World.Calendar.SunPosition.Y < 0) {
				float darkness = (-this.capi.World.Calendar.SunPosition.Y / graceDegrees);

				this.capi.World.Calendar.ReflectColor.R = Math.Max(this.capi.World.Calendar.ReflectColor.R - darkness, NaturalNightSystem.nightRgb[0]);
				this.capi.World.Calendar.ReflectColor.G = Math.Max(this.capi.World.Calendar.ReflectColor.G - darkness, NaturalNightSystem.nightRgb[1]);
				this.capi.World.Calendar.ReflectColor.B = Math.Max(this.capi.World.Calendar.ReflectColor.B - darkness, NaturalNightSystem.nightRgb[2]);

				this.lastRgb[0] = this.capi.World.Calendar.ReflectColor.R;
				this.lastRgb[1] = this.capi.World.Calendar.ReflectColor.G;
				this.lastRgb[2] = this.capi.World.Calendar.ReflectColor.B;
			}
		}
	}
}