using System;
using Vintagestory.API.Client;
using Vintagestory.API.Common;

namespace NaturalNight {
	public class NaturalNightConfig {
		public string colour = NaturalNightSystem.defaultHex;
	}

	public class NaturalNightSystem: ModSystem {
		public const string configFilename = "naturalnight.json";
		public const string defaultHex = "#001022";

		public static float[] nightRgb = new float[3];

		public static NaturalNightConfig config;

		protected ICoreClientAPI capi;
		protected NaturalNightRenderer renderer;

		public override bool ShouldLoad(EnumAppSide forSide) {
			return forSide == EnumAppSide.Client;
		}

		public override void StartClientSide(ICoreClientAPI capi) {
			base.StartClientSide(capi);

			this.capi = capi;
			this.renderer = new NaturalNightRenderer(this.capi);

			this.capi.Event.RegisterRenderer(this.renderer, EnumRenderStage.Before);

			// Load or create config file.
			try {
				NaturalNightSystem.config = this.capi.LoadModConfig<NaturalNightConfig>(configFilename);
			} catch(Exception) {}
			if(NaturalNightSystem.config != null) {
				// Save config file in case of missing fields.
				this.capi.StoreModConfig<NaturalNightConfig>(NaturalNightSystem.config, configFilename);
			} else {
				// Create new config file.
				NaturalNightSystem.config = new NaturalNightConfig();
				this.capi.StoreModConfig<NaturalNightConfig>(NaturalNightSystem.config, configFilename);
			}

			try {
				this.convert(NaturalNightSystem.config.colour, ref NaturalNightSystem.nightRgb);
			} catch(Exception) {
				// Loaded colour is invalid, revert to default.
				this.convert(NaturalNightSystem.defaultHex, ref NaturalNightSystem.nightRgb);
			}

			this.capi.ChatCommands.Create("natnight")
					.BeginSubCommand("set")
						.WithDescription("Set colour.")
						.WithArgs(this.capi.ChatCommands.Parsers.Word("hex"))
						.HandleWith(this.cmdSet)
					.EndSubCommand()
					.BeginSubCommand("reset")
						.WithDescription("Reset colour.")
						.HandleWith(this.cmdReset)
					.EndSubCommand();
		}

		public override void Dispose() {
			base.Dispose();

			this.capi.Event.UnregisterRenderer(this.renderer, EnumRenderStage.Before);
		}

		protected TextCommandResult cmdSet(TextCommandCallingArgs args) {
			string hex = args[0] as string;

			try {
				this.convert(hex, ref NaturalNightSystem.nightRgb);
			} catch(Exception) {
				return TextCommandResult.Error(print("Invalid hex code - must be in format: #RRGGBB"));
			}

			NaturalNightSystem.config.colour = hex;
			this.capi.StoreModConfig<NaturalNightConfig>(NaturalNightSystem.config, configFilename);
			return TextCommandResult.Success(print("Colour set."));
		}

		protected TextCommandResult cmdReset(TextCommandCallingArgs args) {
			this.convert(NaturalNightSystem.defaultHex, ref NaturalNightSystem.nightRgb);

			NaturalNightSystem.config.colour = NaturalNightSystem.defaultHex;
			this.capi.StoreModConfig<NaturalNightConfig>(NaturalNightSystem.config, configFilename);
			return TextCommandResult.Success(print("Colour reset."));
		}

		protected static string print(string text) {
			return "[Natural Night] " + text;
		}

		private void convert(string hex, ref float[] colour) {
			int r = int.Parse(hex.Substring(1, 2), System.Globalization.NumberStyles.HexNumber);
			int g = int.Parse(hex.Substring(3, 2), System.Globalization.NumberStyles.HexNumber);
			int b = int.Parse(hex.Substring(5, 2), System.Globalization.NumberStyles.HexNumber);

			colour[0] = r / 255f;
			colour[1] = g / 255f;
			colour[2] = b / 255f;
		}
	}
}